@@ -514,9 +514,7 @@ static char *
 build_cpp_args (const char *path,
 		const char *cpp_args)
 {
-	char *ret;
-	const char *base;
-	char *base_cpy;
+	char *ret, *base, *base_cpy;
 	int   i;
 
 	base = g_path_get_basename (path);
